/*****************************************************************************
 * Copyright (C) 2009-2016   this file is part of the NPTool Project         *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: matta@lpccaen.in2p3.fr    *
 *                                                                           *
 * Creation Date  : november 2009                                            *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class hold quadranMSQ25  Physics                                            *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
//  NPL
#include "TquadranMSQ25Physics.h"
#include "RootOutput.h"
#include "RootInput.h"
#include "NPDetectorFactory.h"
#include "NPOptionManager.h"
//  STL
#include <iostream>
#include <sstream>
#include <fstream>
#include <limits>
#include <stdlib.h>
using namespace std;
using namespace quadranMSQ25_LOCAL;
//  ROOT
#include "TChain.h"

ClassImp(TquadranMSQ25Physics)
  ///////////////////////////////////////////////////////////////////////////
  TquadranMSQ25Physics::TquadranMSQ25Physics(){    
    NumberOfDetector = 0;
    EventData = new TquadranMSQ25Data;
    PreTreatedData = new TquadranMSQ25Data;
    EventPhysics = this;
    m_E_Threshold = 0;
    m_Pedestal_Threshold = 0;
  }
///////////////////////////////////////////////////////////////////////////
TquadranMSQ25Physics::~TquadranMSQ25Physics()
{}
///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::Clear(){
  DetectorNumber  .clear() ;
  StripNumber     .clear() ;
  Energy          .clear() ;
  Time            .clear() ;
}
///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::ReadConfiguration(NPL::InputParser parser) {
  vector<NPL::InputBlock*> blocks = parser.GetAllBlocksWithToken("quadranMSQ25");
  if(NPOptionManager::getInstance()->GetVerboseLevel())
    cout << "//// " << blocks.size() << " detectors found " << endl; 

  vector<string> cart = {"A","B","C","D"};
  vector<string> sphe = {"R","THETA","PHI","BETA"};

  for(unsigned int i = 0 ; i < blocks.size() ; i++){
    if(blocks[i]->HasTokenList(cart)){
      if(NPOptionManager::getInstance()->GetVerboseLevel())
        cout << endl << "////  quadranMSQ25 " << i+1 <<  endl;
      TVector3 A = blocks[i]->GetTVector3("A","mm");
      TVector3 B = blocks[i]->GetTVector3("B","mm");
      TVector3 C = blocks[i]->GetTVector3("C","mm");
      TVector3 D = blocks[i]->GetTVector3("D","mm");
      NumberOfDetector++;
    }
    else if(blocks[i]->HasTokenList(sphe)){
      if(NPOptionManager::getInstance()->GetVerboseLevel())
        cout << endl << "////  quadranMSQ25 " << i+1 <<  endl;
      double R = blocks[i]->GetDouble("R","mm");
      double Theta = blocks[i]->GetDouble("THETA","deg");
      double Phi = blocks[i]->GetDouble("PHI","deg");
      vector<double> Beta = blocks[i]->GetVectorDouble("BETA","deg");
      NumberOfDetector++;
    }
    else{
      cout << "ERROR: check your input file formatting " << endl;
      exit(1);
    }
  }

  InitializeStandardParameter() ;
  ReadAnalysisConfig()          ;
}


///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::AddParameterToCalibrationManager(){
  CalibrationManager* Cal = CalibrationManager::getInstance();

  for(int i = 0 ; i < NumberOfDetector ; ++i){

    for( int j = 0 ; j < 16 ; ++j){
      Cal->AddParameter("quadranMSQ25", "Detector"+ NPL::itoa(i+1)+"_Strip"+ NPL::itoa(j+1)+"_E","quadranMSQ25_DETECTOR_"+ NPL::itoa(i+1)+"_STRIP_"+ NPL::itoa(j+1)+"_E")  ;
      Cal->AddParameter("quadranMSQ25", "Detector"+ NPL::itoa(i+1)+"_Strip"+ NPL::itoa(j+1)+"_T","quadranMSQ25_DETECTOR_"+ NPL::itoa(i+1)+"_STRIP_"+ NPL::itoa(j+1)+"_T")  ;  
    }
  }
}

///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::InitializeRootInputRaw(){
  TChain* inputChain = RootInput::getInstance()->GetChain();
  inputChain->SetBranchStatus ( "quadranMSQ25"     , true );
  inputChain->SetBranchStatus ( "fquadranMSQ25_*"  , true );
  inputChain->SetBranchAddress( "quadranMSQ25"     , &EventData );
}     
///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::InitializeRootInputPhysics(){
  TChain* inputChain = RootInput::getInstance()->GetChain();
  inputChain->SetBranchStatus ( "quadranMSQ25"          , true );
  inputChain->SetBranchStatus ( "DetectorNumber", true );
  inputChain->SetBranchStatus ( "StripNumber"   , true );
  inputChain->SetBranchStatus ( "Energy"        , true );
  inputChain->SetBranchStatus ( "Time"          , true );
  inputChain->SetBranchAddress( "quadranMSQ25"          , &EventPhysics );

}
///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::InitializeRootOutput(){
  TTree* outputTree = RootOutput::getInstance()->GetTree()      ;
  outputTree->Branch( "quadranMSQ25" , "TquadranMSQ25Physics" , &EventPhysics )  ;
}

///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::BuildPhysicalEvent(){
  BuildSimplePhysicalEvent()  ;
}

///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::BuildSimplePhysicalEvent(){
  PreTreat();
  for(unsigned int i = 0 ; i <   PreTreatedData->GetEnergyMult() ; ++i){
    DetectorNumber  .push_back(   PreTreatedData->GetEnergyDetectorNbr(i) )  ;
    StripNumber     .push_back(   PreTreatedData->GetEnergyStripNbr(i)    )  ;
    Energy          .push_back(   PreTreatedData->GetEnergy(i)            )  ; 
    // Look For associate Time
    for(unsigned int j = 0 ; j <   PreTreatedData->GetTimeMult() ; ++j ){
      if(PreTreatedData->GetEnergyDetectorNbr(i) == PreTreatedData->GetTimeDetectorNbr(j) && PreTreatedData->GetEnergyStripNbr(i)==PreTreatedData->GetTimeStripNbr(j))
        Time.push_back(PreTreatedData->GetTime(j));
    }                        
  }
  return;    
}

///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::PreTreat(){
  ClearPreTreatedData();

  //  E
  for(int i = 0 ; i < EventData->GetEnergyMult() ; ++i){
    if(EventData->GetEnergy(i) > m_Pedestal_Threshold && ChannelStatus[EventData->GetEnergyDetectorNbr(i)-1][EventData->GetEnergyStripNbr(i)-1]){
      double E = fSi_E(EventData , i); 
      if( E > m_E_Threshold ){
        PreTreatedData->SetEnergyDetectorNbr( EventData->GetEnergyDetectorNbr(i) )  ;
        PreTreatedData->SetEnergyStripNbr( EventData->GetEnergyStripNbr(i) )        ;
        PreTreatedData->SetEnergy( E )                                              ;
      }
    } 
  }
  //  T
  for(int i = 0 ; i < EventData->GetTimeMult() ; ++i){
    if(ChannelStatus[EventData->GetTimeDetectorNbr(i)-1][EventData->GetTimeStripNbr(i)-1]){
      PreTreatedData->SetTimeDetectorNbr( EventData->GetTimeDetectorNbr(i) )  ;
      PreTreatedData->SetTimeStripNbr( EventData->GetTimeStripNbr(i) )        ;
      PreTreatedData->SetTime( fSi_T(EventData , i) )                         ;
    }
  }
}

///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::InitializeStandardParameter(){
  //  Enable all channel
  vector<bool> TempChannelStatus;
  ChannelStatus.clear();
  TempChannelStatus.resize(16,true);
  for(int i = 0 ; i < NumberOfDetector ; ++i)   
    ChannelStatus[i] = TempChannelStatus;
}
///////////////////////////////////////////////////////////////////////////
void TquadranMSQ25Physics::ReadAnalysisConfig(){
  bool ReadingStatus = false;

  // path to file
  string FileName = "./configs/ConfigquadranMSQ25.dat";

  // open analysis config file
  ifstream AnalysisConfigFile;
  AnalysisConfigFile.open(FileName.c_str());

  if (!AnalysisConfigFile.is_open()) {
    cout << " No ConfigMust2.dat found: Default parameter loaded for Analayis " << FileName << endl;
    return;
  }
  cout << " Loading user parameter for Analysis from ConfigquadranMSQ25.dat " << endl;

  // Save it in a TAsciiFile
  TAsciiFile* asciiConfig = RootOutput::getInstance()->GetAsciiFileAnalysisConfig();
  asciiConfig->AppendLine("%%% ConfigquadranMSQ25.dat %%%");
  asciiConfig->Append(FileName.c_str());
  asciiConfig->AppendLine("");

  // read analysis config file
  string LineBuffer,DataBuffer,whatToDo;
  while (!AnalysisConfigFile.eof()) {
    // Pick-up next line
    getline(AnalysisConfigFile, LineBuffer);

    // search for "header"
    if (LineBuffer.compare(0, 11, "ConfigquadranMSQ25") == 0) ReadingStatus = true;

    // loop on tokens and data
    while (ReadingStatus) {

      whatToDo= "" ;
      AnalysisConfigFile >> whatToDo;

      // Search for comment symbol (%)
      if (whatToDo.compare(0, 1, "%") == 0) {
        AnalysisConfigFile.ignore(numeric_limits<streamsize>::max(), '\n' );
      }

      else if (whatToDo=="PEDESTAL_THRESHOLD") {
        AnalysisConfigFile >> DataBuffer;
        m_Pedestal_Threshold = atof(DataBuffer.c_str() );
        cout << "PEDESTAL THRESHOLD  " << m_Pedestal_Threshold << endl;
      }


      else if (whatToDo=="DISABLE_ALL") {
        AnalysisConfigFile >> DataBuffer;
        cout << whatToDo << "  " << DataBuffer << endl;
        int Detector = atoi(DataBuffer.substr(2,1).c_str());
        vector< bool > ChannelStatusBuffer;
        ChannelStatusBuffer.resize(16,false);
        ChannelStatus[Detector-1] = ChannelStatusBuffer;
      }

      else if (whatToDo=="DISABLE_CHANNEL") {
        AnalysisConfigFile >> DataBuffer;
        cout << whatToDo << "  " << DataBuffer << endl;
        int telescope = atoi(DataBuffer.substr(2,1).c_str());
        int channel = -1;
        if (DataBuffer.compare(3,3,"STR") == 0) {
          channel = atoi(DataBuffer.substr(6).c_str());
          *(ChannelStatus[telescope-1].begin()+channel-1) = false;
        }

        else {
          cout << "Warning: detector type for quadranMSQ25 unknown!" << endl;
        }
      }

      else {
        ReadingStatus = false;
      }
    }
  }
}   


///////////////////////////////////////////////////////////////////////////
double quadranMSQ25_LOCAL::fSi_E( const TquadranMSQ25Data* EventData , const int i ){

  static string str;
  str = "quadranMSQ25/Detector" + NPL::itoa( EventData->GetEnergyDetectorNbr(i) ) + "_Strip" + NPL::itoa( EventData->GetEnergyStripNbr(i) ) +"_E";
  return CalibrationManager::getInstance()->ApplyCalibration( str, EventData->GetEnergy(i) );
}


double quadranMSQ25_LOCAL::fSi_T( const TquadranMSQ25Data* EventData , const int i ){
  static string str;
  str = "quadranMSQ25/Detector" + NPL::itoa( EventData->GetEnergyDetectorNbr(i) ) + "_Strip" + NPL::itoa( EventData->GetEnergyStripNbr(i) ) +"_T";
  return CalibrationManager::getInstance()->ApplyCalibration( str, EventData->GetTime(i) );
}  



////////////////////////////////////////////////////////////////////////////////
//            Construct Method to be pass to the DetectorFactory              //
////////////////////////////////////////////////////////////////////////////////
NPL::VDetector* TquadranMSQ25Physics::Construct(){
  return (NPL::VDetector*) new TquadranMSQ25Physics();
}

////////////////////////////////////////////////////////////////////////////////
//            Registering the construct method to the factory                 //
////////////////////////////////////////////////////////////////////////////////
extern "C"{
class proxy_quadranmsq25{
  public:
    proxy_quadranmsq25(){
      NPL::DetectorFactory::getInstance()->AddToken("quadranMSQ25","quadranMSQ25");
      NPL::DetectorFactory::getInstance()->AddDetector("quadranMSQ25",TquadranMSQ25Physics::Construct);
    }
};

proxy_quadranmsq25 p;
}

